/* Afvalalarm V12 */
/* CC BY-NC-SA Jeroen Brinkman */
/* Versie 12: dd 15-10-2024 */

#include "RTClib.h" // zie:https://github.com/adafruit/RTClib 

RTC_DS1307 rtc;

/* LETOP: Zet ZETKLOK alleen op true om de tijd eenmalig juist te zetten. Compileer en upload daarna opnieuw met ZETKLOK is false */
#define ZETKLOK false  // Zet deze eenmalig op true als de datum en/of tijd moet worden aangepast en upload de code. Het alarm zal alleen goed werken als deze op false staat
  
/* Dit stuk komt uit de EXCEL sheet */
#define JAAR 2024           // Het jaar van de afvalkalender
const int afval[13][31] = { // dit is de afvalkalender van 2024 met per rij de 31 dagen
{0,0,0,4,3,0,0,0,0,0,0,1,0,2,0,0,0,0,3,0,0,0,0,0,0,0,1,0,0,0,0},  // dec 2023
{0,34,0,2,0,0,0,0,1,0,0,0,0,0,0,3,0,0,0,0,0,0,1,0,2,0,0,0,4,3,0},  // jan
{0,0,0,0,0,1,0,0,0,0,0,0,3,0,2,0,0,0,0,1,0,0,0,0,0,4,3,0,0,0,0},  // feb
{0,0,0,0,1,0,2,0,0,0,0,3,0,0,0,0,0,0,1,0,0,0,0,0,4,3,0,2,0,0,0},  // mrt
{0,1,0,0,0,0,0,0,3,0,0,0,0,0,0,1,0,2,0,0,0,4,3,0,0,0,0,0,0,1,0},  // apr
{0,0,0,0,0,0,3,2,0,0,0,0,0,1,0,0,0,0,0,0,34,0,0,0,0,0,0,1,0,2,0},  // mei
{0,0,0,3,0,0,0,0,0,0,1,0,0,0,0,0,4,3,0,2,0,0,0,0,1,0,0,0,0,0,0},  // jun
{0,3,0,0,0,0,0,0,1,0,2,0,0,0,4,3,0,0,0,0,0,0,1,0,0,0,0,0,0,3,0},  // jul
{2,0,0,0,0,1,0,0,0,0,0,4,3,0,0,0,0,0,0,1,0,2,0,0,0,0,3,0,0,0,0},  // aug
{0,0,1,0,0,0,0,0,4,3,0,2,0,0,0,0,1,0,0,0,0,0,0,3,0,0,0,0,0,0,0},  // sep
{1,0,2,0,0,0,4,3,0,0,0,0,0,0,1,0,0,0,0,0,0,3,0,2,0,0,0,0,1,0,0},  // okt
{0,0,0,4,3,0,0,0,0,0,0,1,0,2,0,0,0,0,3,0,0,0,0,0,0,1,0,0,0,0,0},  // nov
{0,4,3,0,2,0,0,0,0,1,0,0,0,0,0,0,3,0,0,0,0,0,0,1,0,0,2,0,0,4,3}};  // dec
/* Einde stuk uit de EXCEL sheet */

/* pas de volgende uren aan wanneer dat gewenst is */
#define AAN1     16      // Welk uur op de dag voor de afval ophaaldag moet de LED aan
#define UIT1     23      // En op welk uur weer uit
#define AAN2      6      // Welk uur op de afval ophaaldag moet de LED aan
#define UIT2     10      // En op welk uur weer uit

#define UIT       4      // Constante met daarin de poort voor de eerste poort  
#define USEDST true      // Use DST (true or false)
#define BAUD 115200      // Baud rate for the serial monitor

#define REBOOT    4      // Poort gebruikt om opnieuw op te starten 
#define WIT       5      // Witte LED - restafval       - 1 in de matrix LET OP: de poorten van de leds moeten opeenvolgend zijn en mogen 13 niet omvatten
#define ORANJE    6      // Gele/Oranje LED - PMD afval - 2 in de matrix
#define GROEN     7      // Groene LED - GFT afval      - 3 in de matrix
#define BLAUW     8      // Blauwe LED - oud papier     - 4 in de matrix
#define ALIVE     9      // LED die laat zien dat de klok nog loopt
#define HEART    13      // On-board heartbeat

int TeDoen, TeDoen2, dag, maand, jaar, minuut, correctie, lastsec;
DateTime now, vuil;
unsigned long nu, check1, check2, lastmil;

// DATUM & TIJD ==============================================================================================================================================================

DateTime dstclock(DateTime n) { // Return the given (DST adjusted) date and time according to DST settings (for extensive date and time calculations)
  
  DateTime b, e;
  
  b = DateTime(n.year(), 3, 31, 2, 0, 0); // Begin of DST: set on March 31 2:00 (am)
  if (n.month() == 3) b = DateTime(n.year(), 3, 31 - b.dayOfTheWeek(), 2, 0, 0); // Begin of DST: adjusted to last sunday in March 2:00 (am) when actual month is March
  e = DateTime(n.year(), 10, 31, 2, 0, 0); // End of DST: set on October 31 2:00 (am)
  if (n.month() == 10) e = DateTime(n.year(), 10, 31 - e.dayOfTheWeek(), 2, 0, 0); // End of DST: adjusted to last sunday in October 2:00 (am) when actual month is October

  if (USEDST && (n >= b) && (n < e)) n = n + TimeSpan(0,1,0,0); // adjust to standard time if within summertime
  return n;
}; 

DateTime getclock() { // Retrieve the (DST adjusted) date and time
  return dstclock(rtc.now());
}; 

void setclock(DateTime n) { // if the clock is set during summertime then adjust the clock to standard time

  if (USEDST && (n != dstclock(n))) n = n - TimeSpan(0,1,0,0); // if summertime then adjust to the standard time 
  rtc.adjust(n); // Set the clock to standard time
}; 

// SETUP ==============================================================================================================================================================
void setup() {

  /* define I/O  */
  digitalWrite(REBOOT, HIGH); 
  delay(200);
  pinMode(REBOOT, OUTPUT);
  digitalWrite(REBOOT, HIGH); 

  pinMode(WIT, OUTPUT);
  pinMode(ORANJE, OUTPUT);
  pinMode(GROEN, OUTPUT);
  pinMode(BLAUW, OUTPUT);
  pinMode(ALIVE, OUTPUT);
  pinMode(HEART,  OUTPUT); // blinking led showing the programma's hartbeat
    
  /* Check de LED's */
  digitalWrite(WIT, HIGH);
  delay(500);
  digitalWrite(WIT, LOW); digitalWrite(ORANJE, HIGH);
  delay(500);
  digitalWrite(ORANJE, LOW); digitalWrite(GROEN, HIGH);
  delay(500);   
  digitalWrite(GROEN, LOW); digitalWrite(BLAUW, HIGH);
  delay(500);   
  digitalWrite(BLAUW, LOW);
  delay(500); 
     
  /* initialiseer de rtc */
  rtc.begin();
  now = getclock(); 

  /* initialiseer de seriele poort */
  Serial.begin(BAUD);
  Wire.begin();
  Serial.println(""); Serial.println("--------- Begin ---------");
  Serial.print("Huidige datum: "); Serial.print(now.day(), DEC); Serial.print("/"); Serial.print(now.month(), DEC); Serial.print("/"); Serial.println(now.year(), DEC);  
  Serial.print("Huidige tijd: "); Serial.print(now.hour(), DEC); Serial.print(":"); if (now.minute() < 10) Serial.print("0"); Serial.println(now.minute(), DEC);

  if (ZETKLOK) {

    if (!rtc.isrunning()) {
      Serial.println("RTC loopt niet!");
    }

    Serial.println("RTC wordt op de juiste tijd gezet");
    setclock(DateTime(F(__DATE__), F(__TIME__)));
    now = getclock(); 
    Serial.print("Nieuwe datum: "); Serial.print(now.day(), DEC); Serial.print("/"); Serial.print(now.month(), DEC); Serial.print("/"); Serial.println(now.year(), DEC);
    Serial.print("Nieuwe tijd: "); Serial.print(now.hour(), DEC); Serial.print(":"); if (now.minute() < 10) Serial.print("0"); Serial.println(now.minute(), DEC);
  
    /* print de afhaaldata */
    Serial.println("");
    Serial.println("Gevonden afhaaldata");
    TeDoen = 0; maand = 0; dag = 1;
    while (maand != 13){
      TeDoen = afval[(maand)][(dag - 1)];
      if (TeDoen != 0) {
	    if (dag < 10) Serial.print(" "); Serial.print(dag); Serial.print("/"); 
        if ((maand < 10) && (maand > 0)) Serial.print("0"); (maand == 0) ? Serial.print("12") : Serial.print(maand); Serial.print("/"); 
        (maand == 0) ? Serial.print(JAAR-1) : Serial.print(JAAR); Serial.print(" -> ");
        switch (TeDoen) {
        case 1:
          Serial.print("Grijze bak - restafval");
          break;
        case 2:
          Serial.print("Oranje bak - PMT afval");
          break;
        case 3:
          Serial.print("Groene bak - GFT afval");
          break;
        case 4:
          Serial.print("Blauwe bak - Papier en karton");
          break;
        }
        switch (TeDoen/10) {
        case 1:
          Serial.print("Grijze bak - restafval");
          break;
        case 2:
          Serial.print("Oranje bak - PMT afval");
          break;
        case 3:
          Serial.print("Groene bak - GFT afval");
          break;
        case 4:
          Serial.print("Blauwe bak - Papier en karton");
          break;
        }
        if (TeDoen > 10) {
          TeDoen = TeDoen/10; // splits de containers
          switch (TeDoen) {
          case 1:
            Serial.print(" & Grijze bak - restafval");
            break;
          case 2:
            Serial.print(" & Oranje bak - PMT afval");
            break;
          case 3:
            Serial.print(" & Groene bak - GFT afval");
            break;
          case 4:
            Serial.print(" & Blauwe bak - Papier en karton");
            break;
          }
        }
        Serial.println("");
      }
      dag = dag + 1; 
      if (dag == 32) {   
        maand = maand + 1; 
        dag = 1;
      } 
    }

  }
  digitalWrite(WIT, HIGH); digitalWrite(ORANJE, HIGH); digitalWrite(GROEN, HIGH); digitalWrite(BLAUW, HIGH); digitalWrite(ALIVE, HIGH);
  delay(5000); // Wait 10 seconds before activating the watchdog to be sure that new software can be loaded
  digitalWrite(WIT, LOW); digitalWrite(ORANJE, LOW); digitalWrite(GROEN, LOW); digitalWrite(BLAUW, LOW); digitalWrite(ALIVE, LOW);
}

// MAIN ==============================================================================================================================================================

void loop() {
  digitalWrite(HEART, (millis() / 1000) % 2); //1s heartbeat voor de Arduino LED
  now = getclock(); 
  
  /* loopt de klok nog */ 
  if (!rtc.isrunning()) digitalWrite(REBOOT, LOW); // RTC is not running, wait three seconds and therewith initiate the watchdog reset.
  if (lastsec != now.second()){ // the clock has advanced a second
    lastmil = millis() + 2000; // two seconds from now must never occur
    lastsec = now.second();
  } 
  if (millis() > lastmil) digitalWrite(REBOOT, LOW); // if the clock has not advanced within 2 seconds, wait three seconds and therewith initiate the watchdog reset.
  if ((now.hour() == 2) && (now.minute() == 2) && (now.second() == 2)) digitalWrite(REBOOT, LOW); // reset het alarm every night at 02:02:02
 
  TeDoen = 0; TeDoen2 = 0; 
  maand = now.month(); if (now.year() == (JAAR - 1)) maand = 0; // zet de maand op 0 als het vorig jaar is
 
  /* Moet de LED op de vuilophaaldag aan */
  if (now.year() == JAAR) TeDoen = afval[maand][(now.day() - 1)]; // Check of er afval aan de weg moet staan, het array begint bij 0
  if ((nu < AAN2) || (nu >= UIT2)) TeDoen = 0; // Als de LED niet moet branden, zet hem dan weer op nul
  
  /* Moet de LED op de dag voor de vuilophaaldag aan */
  vuil = (now.unixtime() + 86400L); // een dag later
  maand = vuil.month(); if ((maand == 12) && (vuil.year() == (JAAR - 1))) maand = 0; // december JAAR-1 zit in rij 0
  if (TeDoen == 0) { // Als de led niet brandt voor vandaag, kijk dan of hij voor morgen moet branden
    TeDoen = afval[maand][(vuil.day() - 1)]; // Check of er afval aan de weg moet staan, het array begint bij 0
    if ((nu < AAN1) || (nu >= UIT1)) TeDoen = 0; // Als de LED niet moet branden, zet hem dan weer op nul
  }

  /* Bepaal of er een dubbele ophaaldag is */
  if (TeDoen > 10) {
    TeDoen2 = TeDoen/10; // splits de containers
    TeDoen = TeDoen - (TeDoen2*10);
  }

  /* zet de led aan of uit */
  if (TeDoen == 0) { 
    digitalWrite(WIT, LOW); // zet de leds uit
    digitalWrite(ORANJE, LOW);
    digitalWrite(GROEN, LOW);
    digitalWrite(BLAUW, LOW);
  } else { 
    digitalWrite((UIT + TeDoen), HIGH); // zet de betreffende led aan
    if (TeDoen2 > 0) digitalWrite((UIT + TeDoen2), HIGH); // zet de betreffende led aan
  } 
       
  if (!rtc.isrunning()) { //alle LED's gaan knipperen wanneer de klok stuk is
    digitalWrite(WIT, (millis() / 1000) % 2); 
    digitalWrite(ORANJE, (millis() / 1000) % 2); 
    digitalWrite(GROEN, (millis() / 1000) % 2); 
    digitalWrite(BLAUW, (millis() / 1000) % 2);  
  }
  
  /* laat zien dat de klok loopt */
  if (minuut != now.minute()) {
    if (ZETKLOK) {
      Serial.print("Actuele tijd: "); Serial.print(nu, DEC); Serial.print(":"); if (now.minute() < 10) Serial.print("0"); Serial.println(now.minute(), DEC);
    }
    minuut = now.minute();
    digitalWrite(ALIVE, HIGH); delay(100); digitalWrite(ALIVE, LOW);
  }
}
